@@ -137,18 +137,32 @@ def intercept_new_message(process_func):
                 new_message=new_message,
                 **kwargs,
             ):
+                if event is None:
+                    logger.error(f"Event is None with new_message: {new_message}")
+                    continue
+
                 yield event
+                event_metadata = f"| agent_name: {event.author} , user_id: {user_id} , session_id: {session_id} , invocation_id: {event.invocation_id}"
                 if event.get_function_calls():
                     for function_call in event.get_function_calls():
-                        logger.debug(f"Function call: {function_call}")
-                elif (
-                    event.content is not None
-                    and event.content.parts
-                    and event.content.parts[0].text is not None
-                    and len(event.content.parts[0].text.strip()) > 0
-                ):
-                    final_output = event.content.parts[0].text
-                    logger.debug(f"Event output: {final_output}")
+                        logger.debug(f"Function call: {function_call} {event_metadata}")
+                elif event.get_function_responses():
+                    for function_response in event.get_function_responses():
+                        logger.debug(
+                            f"Function response: {function_response} {event_metadata}"
+                        )
+                elif event.content is not None and event.content.parts:
+                    for part in event.content.parts:
+                        if len(part.text.strip()) > 0:
+                            final_output = part.text
+                            if part.thought:
+                                logger.debug(
+                                    f"Thinking output: {final_output} {event_metadata}"
+                                )
+                            else:
+                                logger.debug(
+                                    f"Event output: {final_output} {event_metadata}"
+                                )
 
             post_run_process(self)
 
@@ -303,9 +317,9 @@ class Runner(ADKRunner):
 
     def __init__(
         self,
-        agent: BaseAgent | Agent,
+        agent: BaseAgent | Agent | None = None,
         short_term_memory: ShortTermMemory | None = None,
-        app_name: str = "veadk_default_app",
+        app_name: str | None = None,
         user_id: str = "veadk_default_user",
         upload_inline_data_to_tos: bool = False,
         run_processor: "BaseRunProcessor | None" = None,
@@ -324,15 +338,15 @@ class Runner(ADKRunner):
             agent (google.adk.agents.base_agent.BaseAgent | veadk.agent.Agent):
                 The agent instance used to run interactions.
             short_term_memory (ShortTermMemory | None): Optional short-term memory; if
-                not provided and no external ``session_service`` is supplied, an in-memory
+                not provided and no external `session_service` is supplied, an in-memory
                 session service will be created.
-            app_name (str): Application name. Defaults to ``"veadk_default_app"``.
-            user_id (str): Default user ID. Defaults to ``"veadk_default_user"``.
-            upload_inline_data_to_tos (bool): Whether to enable inline media upload. Defaults to ``False``.
+            app_name (str): Application name. Defaults to `veadk_default_app`.
+            user_id (str): Default user ID. Defaults to `veadk_default_user`.
+            upload_inline_data_to_tos (bool): Whether to enable inline media upload. Defaults to `False`.
             run_processor (BaseRunProcessor | None): Optional run processor for intercepting agent execution.
                 If not provided, will try to get from agent. If agent doesn't have one, uses NoOpRunProcessor.
-            *args: Positional args passed through to ``ADKRunner``.
-            **kwargs: Keyword args passed through to ``ADKRunner``; may include
+            *args: Positional args passed through to `ADKRunner`.
+            **kwargs: Keyword args passed through to `ADKRunner`; may include
                 ``session_service`` and ``memory_service`` to override defaults.
 
         Returns:
@@ -393,6 +407,10 @@ class Runner(ADKRunner):
             else:
                 logger.info("No long term memory provided.")
 
+        # For forward compatibility, we pass app_name to ADKRunner.
+        if not kwargs.get("app") and not app_name:
+            app_name = "veadk_default_app"
+
         super().__init__(
             agent=agent,
             session_service=session_service,
@@ -454,6 +472,11 @@ class Runner(ADKRunner):
             )
         logger.info(f"Run config: {run_config}")
 
+        if self.agent.skills:
+            from veadk.tools.skills_tools.session_path import initialize_session_path
+
+            initialize_session_path(session_id)
+
         user_id = user_id or self.user_id
 
         converted_messages: list = _convert_messages(
@@ -488,17 +511,15 @@ class Runner(ADKRunner):
                         yield event
 
                 async for event in event_generator():
-                    if event.get_function_calls():
-                        for function_call in event.get_function_calls():
-                            logger.debug(f"Function call: {function_call}")
-                    elif (
-                        event.content is not None
-                        and event.content.parts
-                        and event.content.parts[0].text is not None
-                        and len(event.content.parts[0].text.strip()) > 0
-                    ):
-                        final_output = event.content.parts[0].text
-                        logger.debug(f"Event output: {final_output}")
+                    if event.content is not None and event.content.parts:
+                        for part in event.content.parts:
+                            if (
+                                not part.thought
+                                and part.text
+                                and len(part.text.strip()) > 0
+                            ):
+                                final_output = part.text
+                                break
             except LlmCallsLimitExceededError as e:
                 logger.warning(f"Max number of llm calls limit exceeded: {e}")
                 final_output = ""
@@ -668,7 +689,7 @@ class Runner(ADKRunner):
         return eval_set_path
 
     async def save_session_to_long_term_memory(
-        self, session_id: str, user_id: str = "", app_name: str = ""
+        self, session_id: str, user_id: str = "", app_name: str = "", **kwargs
     ) -> None:
         """Save the specified session to long-term memory.
 
@@ -724,5 +745,5 @@ class Runner(ADKRunner):
             )
             return
 
-        await self.long_term_memory.add_session_to_memory(session)
+        await self.long_term_memory.add_session_to_memory(session, kwargs=kwargs)
         logger.info(f"Add session `{session.id}` to long term memory.")
